﻿using Microsoft.Extensions.Logging;
using PlumbingRepairContracts.BindingModels;
using PlumbingRepairContracts.BusinessLogicsContracts;
using PlumbingRepairContracts.SearchModels;
using PlumbingRepairContracts.StoragesContracts;
using PlumbingRepairContracts.ViewModels;
using PlumbingRepairDataModels.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PlumbingRepairBusinessLogic.BusinessLogics
{
    public class StoreLogic : IStoreLogic
    {
        private readonly ILogger _logger;
        private readonly IStoreStorage _storeStorage;
        public StoreLogic(ILogger<StoreLogic> logger, IStoreStorage storeStorage)
        {
            _logger = logger;
            _storeStorage = storeStorage;
        }
        public bool AddWork(StoreSearchModel model, IWorkModel work, int quantity)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }

            if (quantity <= 0)
            {
                throw new ArgumentException("Количество добавляемого изделия должно быть больше 0", nameof(quantity));
            }

            _logger.LogInformation("AddWorkInStore. StoreName:{StoreName}.Id:{ Id}", model.StoreName, model.Id);
            var element = _storeStorage.GetElement(model);

            if (element == null)
            {
                _logger.LogWarning("AddWorkInStore element not found");
                return false;
            }

            _logger.LogInformation("AddWorkInStore find. Id:{Id}", element.Id);

            if (element.Works.TryGetValue(work.Id, out var pair))
            {
                element.Works[work.Id] = (work, quantity + pair.Item2);
                _logger.LogInformation("AddWorkInStore. Has been added {quantity} {work} in {StoreName}", quantity, work.WorkName, element.StoreName);
            }
            else
            {
                element.Works[work.Id] = (work, quantity);
                _logger.LogInformation("AddWorkInShop. Has been added {quantity} new Work {work} in {StoreName}", quantity, work.WorkName, element.StoreName);
            }

            _storeStorage.Update(new()
            {
                Id = element.Id,
                StoreAdress = element.StoreAdress,
                StoreName = element.StoreName,
                OpeningDate = element.OpeningDate,
                Works = element.Works
            });
            return true;
        }

        public bool Create(StoreBindingModel model)
        {
            CheckModel(model);
            model.Works = new();

            if (_storeStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }

            return true;
        }

        public bool Delete(StoreBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);

            if (_storeStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }

            return true;
        }

        public StoreViewModel? ReadElement(StoreSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }

            _logger.LogInformation("ReadElement. StoreName:{StoreName}.Id:{ Id}", model.StoreName, model.Id);
            var element = _storeStorage.GetElement(model);

            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }

            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }

        public List<StoreViewModel>? ReadList(StoreSearchModel? model)
        {
            _logger.LogInformation("ReadList. StoreName:{StoreName}.Id:{ Id} ", model?.StoreName, model?.Id);

            var list = (model == null) ? _storeStorage.GetFullList() : _storeStorage.GetFilteredList(model);

            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }

            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        public bool Update(StoreBindingModel model)
        {
            CheckModel(model, false);

            if (string.IsNullOrEmpty(model.StoreName))
            {
                throw new ArgumentNullException("Нет названия магазина", nameof(model.StoreName));
            }

            if (_storeStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }

        private void CheckModel(StoreBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }

            if (!withParams)
            {
                return;
            }

            if (string.IsNullOrEmpty(model.StoreName))
            {
                throw new ArgumentNullException("Нет названия магазина", nameof(model.StoreName));
            }

            _logger.LogInformation("Store. StoreName:{0}.StoreAdress:{1}. Id: {2}", model.StoreName, model.StoreAdress, model.Id);
            var element = _storeStorage.GetElement(new StoreSearchModel
            {
                StoreName = model.StoreName
            });

            if (element != null && element.Id != model.Id && element.StoreName == model.StoreName)
            {
                throw new InvalidOperationException("Магазин с таким названием уже есть");
            }
        }
    }
}
